@@ -5,8 +5,8 @@ Supported devices:
 - EVGA CLC (120 CL12, 240, 280 or 360); modern generic Asetek 690LC
 - NZXT Kraken X (X31, X41 or X61); legacy generic Asetek 690LC
 - NZXT Kraken X (X40 or X60); legacy generic Asetek 690LC
-- Corsair H80i GT, H100i GTX or H110i GTX
-- Corsair H80i v2, H100i v2 or H115i
+- Corsair Hydro H80i GT, H100i GTX or H110i GTX
+- Corsair Hydro H80i v2, H100i v2 or H115i
 
 Copyright (C) 2018–2021  Jonas Malaco and contributors
 
@@ -115,16 +115,16 @@ class _CommonAsetekDriver(UsbDriver):
                     + [alert_temp, interval1, interval2, not blackout, fading,
                        blinking, enable_alert, 0x00, 0x01])
 
-    def _prepare_profile(self, profile, min_duty, max_duty):
+    def _prepare_profile(self, profile, min_duty, max_duty, max_points):
         opt = list(profile)
         size = len(opt)
         if size < 1:
             raise ValueError('at least one PWM point required')
         elif size > _MAX_PROFILE_POINTS:
-            raise ValueError(f'too many PWM points ({size}), only 6 supported')
+            raise ValueError(f'too many PWM points ({size}), only {max_points} supported')
         for i, (temp, duty) in enumerate(opt):
             opt[i] = (temp, clamp(duty, min_duty, max_duty))
-        missing = _MAX_PROFILE_POINTS - size
+        missing = max_points - size
         if missing:
             # Some issues were observed when padding with (0°C, 0%), though
             # they were hard to reproduce.  So far it *seems* that in some
@@ -142,9 +142,9 @@ class _CommonAsetekDriver(UsbDriver):
         Enables the device to send data to the host.
         """
 
-        super().connect(**kwargs)
+        ret = super().connect(**kwargs)
         self._configure_flow_control(clear_to_send=True)
-        return self
+        return ret
 
     def initialize(self, **kwargs):
         """Initialize the device."""
@@ -265,7 +265,7 @@ class Modern690Lc(_CommonAsetekDriver):
     def set_speed_profile(self, channel, profile, **kwargs):
         """Set channel to follow a speed duty profile."""
         mtype, dmin, dmax = _VARIABLE_SPEED_CHANNELS[channel]
-        adjusted = self._prepare_profile(profile, dmin, dmax)
+        adjusted = self._prepare_profile(profile, dmin, dmax, _MAX_PROFILE_POINTS)
         for temp, duty in adjusted:
             _LOGGER.info('setting %s PWM point: (%d°C, %d%%), device interpolated',
                          channel, temp, duty)
@@ -317,28 +317,28 @@ class Legacy690Lc(_CommonAsetekDriver):
         self._data = None
 
     def connect(self, runtime_storage=None, **kwargs):
-        super().connect(**kwargs)
-        ids = f'vid{self.vendor_id:04x}_pid{self.product_id:04x}'
-        loc = f'bus{self.bus}_port{"_".join(map(str, self.port))}'
-
+        ret = super().connect(**kwargs)
         if runtime_storage:
             self._data = runtime_storage
         else:
+            ids = f'vid{self.vendor_id:04x}_pid{self.product_id:04x}'
+            loc = f'bus{self.bus}_port{"_".join(map(str, self.port))}'
             self._data = RuntimeStorage(key_prefixes=[ids, loc, 'legacy'])
+        return ret
 
     def _set_all_fixed_speeds(self):
         self._begin_transaction()
         for channel in ['pump', 'fan']:
             mtype, dmin, dmax = _LEGACY_FIXED_SPEED_CHANNELS[channel]
-            duty = clamp(self._data.load_int(f'{channel}_duty', default=dmax), dmin, dmax)
+            duty = clamp(self._data.load(f'{channel}_duty', of_type=int, default=dmax), dmin, dmax)
             _LOGGER.info('setting %s duty to %d%%', channel, duty)
             self._write([mtype, duty])
         return self._end_transaction_and_read()
 
     def initialize(self, **kwargs):
         super().initialize(**kwargs)
-        self._data.store_int('pump_duty', None)
-        self._data.store_int('fan_duty', None)
+        self._data.store('pump_duty', None)
+        self._data.store('fan_duty', None)
         self._set_all_fixed_speeds()
 
     def get_status(self, **kwargs):
@@ -392,7 +392,7 @@ class Legacy690Lc(_CommonAsetekDriver):
         """Set channel to a fixed speed duty."""
         mtype, dmin, dmax = _LEGACY_FIXED_SPEED_CHANNELS[channel]
         duty = clamp(duty, dmin, dmax)
-        self._data.store_int(f'{channel}_duty', duty)
+        self._data.store(f'{channel}_duty', duty)
         self._set_all_fixed_speeds()
 
     def set_speed_profile(self, channel, profile, **kwargs):
