@@ -95,7 +95,7 @@ the devices must be manually initialized by calling `initialize()`.  This will
 cause all connected fans and LED accessories to be detected, and enable status
 updates.  It is recommended to initialize the devices at every boot.
 
-Copyright (C) 2018–2020  Jonas Malaco, CaseySJ and contributors
+Copyright (C) 2018–2021  Jonas Malaco, CaseySJ and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -104,9 +104,10 @@ import logging
 
 from liquidctl.driver.usb import UsbHidDriver
 from liquidctl.error import NotSupportedByDevice
-from liquidctl.util import clamp, Hue2Accessory, HUE2_MAX_ACCESSORIES_IN_CHANNEL
+from liquidctl.util import clamp, map_direction, Hue2Accessory, \
+                           HUE2_MAX_ACCESSORIES_IN_CHANNEL
 
-LOGGER = logging.getLogger(__name__)
+_LOGGER = logging.getLogger(__name__)
 
 _ANIMATION_SPEEDS = {
     'slowest':  0x0,
@@ -119,6 +120,7 @@ _ANIMATION_SPEEDS = {
 _MIN_DUTY = 0
 _MAX_DUTY = 100
 
+
 class _CommonSmartDeviceDriver(UsbHidDriver):
     """Common functions of Smart Device and Grid drivers."""
 
@@ -128,7 +130,7 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
         self._speed_channels = speed_channels
         self._color_channels = color_channels
 
-    def set_color(self, channel, mode, colors, speed='normal', **kwargs):
+    def set_color(self, channel, mode, colors, speed='normal', direction='forward', **kwargs):
         """Set the color mode.
 
         Only supported by Smart Device V1/V2 and HUE 2 controllers.
@@ -136,22 +138,28 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
 
         if not self._color_channels:
             raise NotSupportedByDevice()
+
+        if 'backwards' in mode:
+            _LOGGER.warning('deprecated mode, move to direction=backward option')
+            mode = mode.replace('backwards-', '')
+            direction = 'backward'
+
         cid = self._color_channels[channel]
         _, _, _, mincolors, maxcolors = self._COLOR_MODES[mode]
         colors = [[g, r, b] for [r, g, b] in colors]
         if len(colors) < mincolors:
-            raise ValueError('Not enough colors for mode={}, at least {} required'
-                             .format(mode, mincolors))
+            raise ValueError(f'not enough colors for mode={mode}, at least {mincolors} required')
         elif maxcolors == 0:
             if colors:
-                LOGGER.warning('too many colors for mode=%s, none needed', mode)
+                _LOGGER.warning('too many colors for mode=%s, none needed', mode)
             colors = [[0, 0, 0]]  # discard the input but ensure at least one step
         elif len(colors) > maxcolors:
-            LOGGER.warning('too many colors for mode=%s, dropping to %i',
-                           mode, maxcolors)
+            _LOGGER.warning('too many colors for mode=%s, dropping to %d',
+                            mode, maxcolors)
             colors = colors[:maxcolors]
+
         sval = _ANIMATION_SPEEDS[speed]
-        self._write_colors(cid, mode, colors, sval)
+        self._write_colors(cid, mode, colors, sval, direction)
 
     def set_fixed_speed(self, channel, duty, **kwargs):
         """Set channel to a fixed speed."""
@@ -161,7 +169,7 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
             selected_channels = {channel: self._speed_channels[channel]}
         for cname, (cid, dmin, dmax) in selected_channels.items():
             duty = clamp(duty, dmin, dmax)
-            LOGGER.info('setting %s duty to %i%%', cname, duty)
+            _LOGGER.info('setting %s duty to %d%%', cname, duty)
             self._write_fixed_duty(cid, duty)
 
     def set_speed_profile(self, channel, profile, **kwargs):
@@ -172,7 +180,7 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
         padding = [0x0]*(self._WRITE_LENGTH - len(data))
         self.device.write(data + padding)
 
-    def _write_colors(self, cid, mode, colors, sval):
+    def _write_colors(self, cid, mode, colors, sval, direction):
         raise NotImplementedError()
 
     def _write_fixed_duty(self, cid, duty):
@@ -203,32 +211,34 @@ class SmartDevice(_CommonSmartDeviceDriver):
         'super-fixed':                   (0x00, 0x00, 0x00, 1, 40),  # independent leds
         'fading':                        (0x01, 0x00, 0x00, 1, 8),
         'spectrum-wave':                 (0x02, 0x00, 0x00, 0, 0),
-        'backwards-spectrum-wave':       (0x02, 0x10, 0x00, 0, 0),
         'marquee-3':                     (0x03, 0x00, 0x00, 1, 1),
         'marquee-4':                     (0x03, 0x00, 0x08, 1, 1),
         'marquee-5':                     (0x03, 0x00, 0x10, 1, 1),
         'marquee-6':                     (0x03, 0x00, 0x18, 1, 1),
-        'backwards-marquee-3':           (0x03, 0x10, 0x00, 1, 1),
-        'backwards-marquee-4':           (0x03, 0x10, 0x08, 1, 1),
-        'backwards-marquee-5':           (0x03, 0x10, 0x10, 1, 1),
-        'backwards-marquee-6':           (0x03, 0x10, 0x18, 1, 1),
         'covering-marquee':              (0x04, 0x00, 0x00, 1, 8),
-        'covering-backwards-marquee':    (0x04, 0x10, 0x00, 1, 8),
         'alternating':                   (0x05, 0x00, 0x00, 2, 2),
         'moving-alternating':            (0x05, 0x08, 0x00, 2, 2),
-        'backwards-moving-alternating':  (0x05, 0x18, 0x00, 2, 2),
         'pulse':                         (0x06, 0x00, 0x00, 1, 8),
         'breathing':                     (0x07, 0x00, 0x00, 1, 8),   # colors for each step
         'super-breathing':               (0x07, 0x00, 0x00, 1, 40),  # one step, independent leds
         'candle':                        (0x09, 0x00, 0x00, 1, 1),
         'wings':                         (0x0c, 0x00, 0x00, 1, 1),
         'super-wave':                    (0x0d, 0x00, 0x00, 1, 40),  # independent ring leds
-        'backwards-super-wave':          (0x0d, 0x10, 0x00, 1, 40),  # independent ring leds
+
+        # deprecated in favor of direction=backward
+        'backwards-spectrum-wave':       (0x02, 0x00, 0x00, 0, 0),
+        'backwards-marquee-3':           (0x03, 0x00, 0x00, 1, 1),
+        'backwards-marquee-4':           (0x03, 0x00, 0x08, 1, 1),
+        'backwards-marquee-5':           (0x03, 0x00, 0x10, 1, 1),
+        'backwards-marquee-6':           (0x03, 0x00, 0x18, 1, 1),
+        'covering-backwards-marquee':    (0x04, 0x00, 0x00, 1, 8),
+        'backwards-moving-alternating':  (0x05, 0x08, 0x00, 2, 2),
+        'backwards-super-wave':          (0x0d, 0x00, 0x00, 1, 40),
     }
 
     def __init__(self, device, description, speed_channel_count, color_channel_count, **kwargs):
         """Instantiate a driver with a device handle."""
-        speed_channels = {'fan{}'.format(i + 1): (i, _MIN_DUTY, _MAX_DUTY)
+        speed_channels = {f'fan{i + 1}': (i, _MIN_DUTY, _MAX_DUTY)
                           for i in range(speed_channel_count)}
         color_channels = {'led': (i)
                           for i in range(color_channel_count)}
@@ -257,12 +267,12 @@ class SmartDevice(_CommonSmartDeviceDriver):
             msg = self.device.read(self._READ_LENGTH)
             num = (msg[15] >> 4) + 1
             state = msg[15] & 0x3
-            status.append(('Fan {}'.format(num), ['—', 'DC', 'PWM'][state], ''))
+            status.append((f'Fan {num}', ['—', 'DC', 'PWM'][state], ''))
             noise.append(msg[1])
             if state:
-                status.append(('Fan {} speed'.format(num), msg[3] << 8 | msg[4], 'rpm'))
-                status.append(('Fan {} voltage'.format(num), msg[7] + msg[8]/100, 'V'))
-                status.append(('Fan {} current'.format(num), msg[10]/100, 'A'))
+                status.append((f'Fan {num} speed', msg[3] << 8 | msg[4], 'rpm'))
+                status.append((f'Fan {num} voltage', msg[7] + msg[8]/100, 'V'))
+                status.append((f'Fan {num} current', msg[10]/100, 'A'))
             if i != 0:
                 continue
             fw = '{}.{}.{}'.format(msg[0xb], msg[0xc] << 8 | msg[0xd], msg[0xe])
@@ -277,11 +287,14 @@ class SmartDevice(_CommonSmartDeviceDriver):
         status.append(('Noise level', round(sum(noise)/len(noise)), 'dB'))
         return sorted(status)
 
-    def _write_colors(self, cid, mode, colors, sval):
+    def _write_colors(self, cid, mode, colors, sval, direction='forward'):
         mval, mod3, mod4, _, _ = self._COLOR_MODES[mode]
         # generate steps from mode and colors: usually each color set by the user generates
         # one step, where it is specified to all leds and the device handles the animation;
         # but in super mode there is a single step and each color directly controls a led
+
+        mod3 += map_direction(direction, 0, 0x10)
+
         if 'super' in mode:
             steps = [list(itertools.chain(*colors))]
         else:
@@ -312,7 +325,11 @@ class SmartDevice2(_CommonSmartDeviceDriver):
             'speed_channel_count': 0,
             'color_channel_count': 2
         }),
-        (0x1e71, 0x2009, None, 'NZXT RGB & Fan Controller (experimental)', {
+        (0x1e71, 0x2009, None, 'NZXT RGB & Fan Controller', {
+            'speed_channel_count': 3,
+            'color_channel_count': 2
+        }),
+        (0x1e71, 0x200e, None, 'NZXT RGB & Fan Controller', {
             'speed_channel_count': 3,
             'color_channel_count': 2
         }),
@@ -323,23 +340,17 @@ class SmartDevice2(_CommonSmartDeviceDriver):
     _WRITE_LENGTH = 64
 
     _COLOR_MODES = {
-        # (mode, size/variant, moving/backwards, min colors, max colors)
+        # (mode, size/variant, moving, min colors, max colors)
         'off':                              (0x00, 0x00, 0x00, 0, 0),
         'fixed':                            (0x00, 0x00, 0x00, 1, 1),
         'super-fixed':                      (0x01, 0x00, 0x00, 1, 40),  # independent leds
         'fading':                           (0x01, 0x00, 0x00, 1, 8),
         'spectrum-wave':                    (0x02, 0x00, 0x00, 0, 0),
-        'backwards-spectrum-wave':          (0x02, 0x00, 0x01, 0, 0),
         'marquee-3':                        (0x03, 0x00, 0x00, 1, 1),
         'marquee-4':                        (0x03, 0x01, 0x00, 1, 1),
         'marquee-5':                        (0x03, 0x02, 0x00, 1, 1),
         'marquee-6':                        (0x03, 0x03, 0x00, 1, 1),
-        'backwards-marquee-3':              (0x03, 0x00, 0x01, 1, 1),
-        'backwards-marquee-4':              (0x03, 0x01, 0x01, 1, 1),
-        'backwards-marquee-5':              (0x03, 0x02, 0x01, 1, 1),
-        'backwards-marquee-6':              (0x03, 0x03, 0x01, 1, 1),
         'covering-marquee':                 (0x04, 0x00, 0x00, 1, 8),
-        'covering-backwards-marquee':       (0x04, 0x00, 0x01, 1, 8),
         'alternating-3':                    (0x05, 0x00, 0x00, 2, 2),
         'alternating-4':                    (0x05, 0x01, 0x00, 2, 2),
         'alternating-5':                    (0x05, 0x02, 0x00, 2, 2),
@@ -348,10 +359,6 @@ class SmartDevice2(_CommonSmartDeviceDriver):
         'moving-alternating-4':             (0x05, 0x01, 0x10, 2, 2),   # byte4: 0x10 = moving
         'moving-alternating-5':             (0x05, 0x02, 0x10, 2, 2),   # byte4: 0x10 = moving
         'moving-alternating-6':             (0x05, 0x03, 0x10, 2, 2),   # byte4: 0x10 = moving
-        'backwards-moving-alternating-3':   (0x05, 0x00, 0x11, 2, 2),   # byte4: 0x11 = moving + backwards
-        'backwards-moving-alternating-4':   (0x05, 0x01, 0x11, 2, 2),   # byte4: 0x11 = moving + backwards
-        'backwards-moving-alternating-5':   (0x05, 0x02, 0x11, 2, 2),   # byte4: 0x11 = moving + backwards
-        'backwards-moving-alternating-6':   (0x05, 0x03, 0x11, 2, 2),   # byte4: 0x11 = moving + backwards
         'pulse':                            (0x06, 0x00, 0x00, 1, 8),
         'breathing':                        (0x07, 0x00, 0x00, 1, 8),   # colors for each step
         'super-breathing':                  (0x03, 0x19, 0x00, 1, 40),  # independent leds
@@ -360,17 +367,29 @@ class SmartDevice2(_CommonSmartDeviceDriver):
         'rainbow-flow':                     (0x0b, 0x00, 0x00, 0, 0),
         'super-rainbow':                    (0x0c, 0x00, 0x00, 0, 0),
         'rainbow-pulse':                    (0x0d, 0x00, 0x00, 0, 0),
-        'backwards-rainbow-flow':           (0x0b, 0x00, 0x01, 0, 0),
-        'backwards-super-rainbow':          (0x0c, 0x00, 0x01, 0, 0),
-        'backwards-rainbow-pulse':          (0x0d, 0x00, 0x01, 0, 0),
         'wings':                            (None, 0x00, 0x00, 1, 1),   # wings requires special handling
+
+        # deprecated in favor of direction=backward
+        'backwards-spectrum-wave':          (0x02, 0x00, 0x00, 0, 0),
+        'backwards-marquee-3':              (0x03, 0x00, 0x00, 1, 1),
+        'backwards-marquee-4':              (0x03, 0x01, 0x00, 1, 1),
+        'backwards-marquee-5':              (0x03, 0x02, 0x00, 1, 1),
+        'backwards-marquee-6':              (0x03, 0x03, 0x00, 1, 1),
+        'covering-backwards-marquee':       (0x04, 0x00, 0x00, 1, 8),
+        'backwards-moving-alternating-3':   (0x05, 0x00, 0x01, 2, 2),
+        'backwards-moving-alternating-4':   (0x05, 0x01, 0x01, 2, 2),
+        'backwards-moving-alternating-5':   (0x05, 0x02, 0x01, 2, 2),
+        'backwards-moving-alternating-6':   (0x05, 0x03, 0x01, 2, 2),
+        'backwards-rainbow-flow':           (0x0b, 0x00, 0x00, 0, 0),
+        'backwards-super-rainbow':          (0x0c, 0x00, 0x00, 0, 0),
+        'backwards-rainbow-pulse':          (0x0d, 0x00, 0x00, 0, 0),
     }
 
     def __init__(self, device, description, speed_channel_count, color_channel_count, **kwargs):
         """Instantiate a driver with a device handle."""
-        speed_channels = {'fan{}'.format(i + 1): (i, _MIN_DUTY, _MAX_DUTY)
+        speed_channels = {f'fan{i + 1}': (i, _MIN_DUTY, _MAX_DUTY)
                           for i in range(speed_channel_count)}
-        color_channels = {'led{}'.format(i + 1): (1 << i)
+        color_channels = {f'led{i + 1}': (1 << i)
                           for i in range(color_channel_count)}
         color_channels['sync'] = (1 << color_channel_count) - 1
         super().__init__(device, description, speed_channels, color_channels, **kwargs)
@@ -428,8 +447,8 @@ class SmartDevice2(_CommonSmartDeviceDriver):
             noise_offset = 56
             for i, _ in enumerate(self._speed_channels):
                 if ((msg[rpm_offset] != 0x0) and (msg[rpm_offset + 1] != 0x0)):
-                    status.append(('Fan {} speed'.format(i + 1), msg[rpm_offset + 1] << 8 | msg[rpm_offset], 'rpm'))
-                    status.append(('Fan {} duty'.format(i + 1), msg[duty_offset + i], '%'))
+                    status.append((f'Fan {i + 1} speed', msg[rpm_offset + 1] << 8 | msg[rpm_offset], 'rpm'))
+                    status.append((f'Fan {i + 1} duty', msg[duty_offset + i], '%'))
                 rpm_offset += 2
             status.append(('Noise level', msg[noise_offset], 'dB'))
 
@@ -448,15 +467,16 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                 return
         assert False, f'missing messages (attempts={self._MAX_READ_ATTEMPTS}, missing={len(parsers)})'
 
-    def _write_colors(self, cid, mode, colors, sval):
-        mval, mod3, mod4, mincolors, maxcolors = self._COLOR_MODES[mode]
+    def _write_colors(self, cid, mode, colors, sval, direction='forward',):
+        mval, mod3, movingFlag, mincolors, maxcolors = self._COLOR_MODES[mode]
+
         color_count = len(colors)
         if maxcolors == 40:
             led_padding = [0x00, 0x00, 0x00]*(maxcolors - color_count)  # turn off remaining LEDs
             leds = list(itertools.chain(*colors)) + led_padding
-            self._write([0x22, 0x10, cid, 0x00] + leds[0:60]) # send first 20 colors to device (3 bytes per color)
+            self._write([0x22, 0x10, cid, 0x00] + leds[0:60])  # send first 20 colors to device (3 bytes per color)
             self._write([0x22, 0x11, cid, 0x00] + leds[60:])  # send remaining colors to device
-            self._write([0x22, 0xA0, cid, 0x00, mval, mod3, 0x00, 0x00, 0x00,
+            self._write([0x22, 0xa0, cid, 0x00, mval, mod3, 0x00, 0x00, 0x00,
                          0x00, 0x64, 0x00, 0x32, 0x00, 0x00, 0x01])
         elif mode == 'wings':  # wings requires special handling
             for [g, r, b] in colors:
@@ -466,7 +486,7 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                 color_lists[0] = [g, r, b] * 8
                 color_lists[1] = [int(x // 2.5) for x in color_lists[0]]
                 color_lists[2] = [int(x // 4) for x in color_lists[1]]
-                for i in range(8):   #  send color scheme first, before enabling wings mode
+                for i in range(8):   # send color scheme first, before enabling wings mode
                     mod = 0x05 if i in [3, 7] else 0x01
                     msg = ([0x22, 0x20, cid, i, 0x04, 0x39, 0x00, mod,
                             0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06,
@@ -475,20 +495,20 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                     self._write(msg + color_lists[i % 4])
                 self._write([0x22, 0x03, cid, 0x08])   # this actually enables wings mode
         else:
-            byte7 = (mod4 & 0x10) >> 4  # sets 'moving' flag for moving alternating modes
-            byte8 = mod4 & 0x01  # sets 'backwards' flag
-            byte9 = mod3 if mval == 0x03 else color_count  #  specifies 'marquee' LED size
-            byte10 = mod3 if mval == 0x05 else 0x00  #  specifies LED size for 'alternating' modes
+            byte7 = movingFlag  # sets 'moving' flag for moving alternating modes
+            byte8 = map_direction(direction, 0, 1)  # sets 'backward' flag
+            byte9 = mod3 if mval == 0x03 else color_count  # specifies 'marquee' LED size
+            byte10 = mod3 if mval == 0x05 else 0x00  # specifies LED size for 'alternating' modes
             header = [0x28, 0x03, cid, 0x00, mval, sval, byte7, byte8, byte9, byte10]
             self._write(header + list(itertools.chain(*colors)))
 
     def _write_fixed_duty(self, cid, duty):
-        msg = [0x62, 0x01, 0x01 << cid, 0x00, 0x00, 0x00] # fan channel passed as bitflag in last 3 bits of 3rd byte
-        msg[cid + 3] = duty # duty percent in 4th, 5th, and 6th bytes for, respectively, fan1, fan2 and fan3
+        msg = [0x62, 0x01, 0x01 << cid, 0x00, 0x00, 0x00]  # fan channel passed as bitflag in last 3 bits of 3rd byte
+        msg[cid + 3] = duty  # duty percent in 4th, 5th, and 6th bytes for, respectively, fan1, fan2 and fan3
         self._write(msg)
 
 
-# backwards compatibility
+# backward compatibility
 NzxtSmartDeviceDriver = SmartDevice
 SmartDeviceDriver = SmartDevice
 SmartDeviceV2Driver = SmartDevice2
